/****************************************************************************
File    : MainFrame.cpp
/*
@(#) #SY# Atari800Win
@(#) #IS# CMainFrame implementation file
@(#) #BY# Richard Lawrence, Harry (modifications and code maintenance)
@(#) #LM# 22.02.2000
*/

/*
Copyright (c) 1998 Richard Lawrence

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version. This program is 
distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. You should have received a copy of the GNU
General Public License along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "StdAfx.h"
#include "Atari800Win.h"
#include "Graphics.h"
#include "DriveDlg.h"
#include "HardwareDlg.h"
#include "FileAssociationsDlg.h"
#include "CartridgeDlg.h"
#include "GraphicsDlg.h"
#include "DriveSmallDlg.h"
#include "GraphicsSmallDlg.h"
#include "SoundDlg.h"
#include "FileSmallDlg.h"
#include "HarddiskDlg.h"
#include "JoystickDlg.h"
#include "ErrorLogDlg.h"
#include "KeyboardDlg.h"
#include "WarningDlg.h"
#include "KeyCodes.h"		// Consists static members
#include "MainFrame.h"

//#include "core.h"			// AtariWin core

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/* MainFrame needs this to pause/restart the Atari thread with F9 */
/*#ifdef SUPPORT_UI
extern BOOL	startThread( void );
#endif*/

/////////////////////////////////////////////////////////////////////////////
// CMainFrame window

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_FILE_READSTATE, OnFileReadState)
	ON_COMMAND(ID_FILE_SAVESTATE, OnFileSaveState)
	ON_COMMAND(ID_FILE_SAVESTATEROMS, OnFileSaveStateRoms)
	ON_COMMAND(ID_FILE_DCMTOATR, OnFileDcmToAtr)
	ON_COMMAND(ID_FILE_VIEWLOG, OnFileViewLog)
	ON_COMMAND(ID_ATARI_CARTRIDGE, OnAtariCartridge)
	ON_COMMAND(ID_ATARI_DISK, OnAtariDisk)
	ON_COMMAND(ID_ATARI_HARDDISK, OnAtariHarddisk)
	ON_COMMAND(ID_ATARI_HARDWARE, OnAtariHardware)
	ON_COMMAND(ID_ATARI_LOADEXE, OnAtarLoadExe)
	ON_COMMAND(ID_ATARI_REBOOT, OnAtariReboot)
	ON_COMMAND(ID_OPTIONS_GRAPHICS, OnOptionsGraphics)
	ON_COMMAND(ID_OPTIONS_SOUND, OnOptionsSound)
	ON_COMMAND(ID_OPTIONS_JOYSTICK, OnOptionsJoystick)
	ON_COMMAND(ID_OPTIONS_KEYBOARD, OnOptionsKeyboard)
	ON_COMMAND(ID_OPTIONS_SOUNDFILE, OnOptionsSoundFile)
	ON_COMMAND(ID_OPTIONS_ARTIF, OnOptionsArtif)
	ON_COMMAND(ID_MISC_BENCHMARK, OnMiscBenchmark)
	ON_COMMAND(ID_MISC_SOUND_POKEY_SPEED, OnMiscSoundPokeySpeed)
	ON_COMMAND(ID_MISC_FILE_ASSOCIATIONS, OnMiscFileAssociations)
	ON_COMMAND(ID_MISC_CLEARALL, OnMiscClearAll)
	ON_COMMAND(ID_MISC_MONITOR, OnMiscMonitor)
	ON_WM_MOVE()
	ON_WM_TIMER()
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
	// Global help commands
	ON_COMMAND(ID_HELP_FINDER, CFrameWnd::OnHelpFinder)
	ON_COMMAND(ID_HELP, CFrameWnd::OnHelp)
	ON_COMMAND(ID_CONTEXT_HELP, CFrameWnd::OnContextHelp)
	ON_COMMAND(ID_DEFAULT_HELP, CFrameWnd::OnHelpFinder)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR // status line indicator
};

/*========================================================
Method   : CMainFrame::CMainFrame
=========================================================*/
/* #FN#
   Standard constructor */
CMainFrame::
CMainFrame()
{
	BuildCRCTable();

	HandleRegistry();
	if( g_ulMiscStates & ATARI_LAST_BOOT_FAILED )
	{
		int nResult;

		LoadString( NULL, IDS_LOCKUP_WARNING, g_tszErrorString, LOADSTRING_STRING_SIZE );
		nResult = MessageBox( g_tszErrorString, "Atari800Win", MB_ICONQUESTION | MB_YESNO );
		if( nResult == IDYES )
		{
			g_ulScreenMode = DISPLAY_MODE_DEFAULT;
			g_ulSoundState = SOUND_NOSOUND;
			if( g_nSoundRate != 22050 )
				g_ulSoundState |= SOUND_CUSTOM_RATE;
			WriteInitialReg( NULL, TRUE );
		}
		g_ulMiscStates &= ~ATARI_LAST_BOOT_FAILED;
	}
	/* Call the main of Atari emulator */
	main( g_argc, g_argv );
} /* #OF# CMainFrame::CMainFrame */

/*========================================================
Method   : CMainFrame::~CMainFrame
=========================================================*/
/* #FN#
   Destructor */
CMainFrame::
~CMainFrame()
{
	/* In case desktop isn't getting refreshed when exiting directly from
	   DirectDraw full screen */
	::InvalidateRect( NULL, NULL, FALSE );
} /* #OF# CMainFrame::~CMainFrame */


/////////////////////////////////////////////////////////////////////////////
// CMainFrame implementation

/*========================================================
Method   : CMainFrame::PreCreateWindow
=========================================================*/
/* #FN#
   Called by the framework before the creation of the Windows window */
BOOL
/* #AS#
   Nonzero if the window creation should continue, 0 to indicate creation
   failure */
CMainFrame::
PreCreateWindow( CREATESTRUCT& cs )
{
	cs.x = g_nStartX;
	cs.y = g_nStartY;

//	cs.style |= FWS_ADDTOTITLE;
//	cs.style |= CS_CLASSDC;
//	cs.style |= CS_PARENTDC;
//	cs.style |= CS_BYTEALIGNCLIENT;
//	cs.style |= CS_BYTEALIGNWINDOW;
	cs.style |= CS_OWNDC;
	cs.style &= ~(WS_THICKFRAME | WS_MAXIMIZEBOX);

	return CFrameWnd::PreCreateWindow( cs );
} /* #OF# CMainFrame::PreCreateWindow */

/*========================================================
Method   : CMainFrame::ActivateFrame
=========================================================*/
/* #FN#
   Activates and restores the frame window */
void
/* #AS#
   Nothing */
CMainFrame::
ActivateFrame( int nCmdShow /* #IN# Parameter to pass to CWnd::ShowWindow */ )
{
	/* Get the real size of status bar */
	CRect rect;
	pStatusBar->GetWindowRect( rect );
	g_nStatusSize = rect.Height();

	CFrameWnd::ActivateFrame( nCmdShow );
	// Causes some problems with window mode switching
	//	(g_ulScreenMode & SM_MODE_FULL) ?
	//	SW_SHOWMAXIMIZED : nCmdShow );
} /* #OF# CMainFrame::ActivateFrame */

/*========================================================
Method   : CMainFrame::DriveInsert
=========================================================*/
/* #FN#
   Inserts disk image to pointed virtual drive */
void
/* #AS#
   Nothing */
CMainFrame::
DriveInsert( int nDriveNum )
{
	CString	strName = "";
	char	szCurDir[ MAX_PATH ];
	char	szNewDir[ MAX_PATH ];
	char	szPrompt[ 128 ];

	sprintf( szPrompt, "Select disk to insert in drive %d", nDriveNum );
	GetCurrentDirectory( MAX_PATH, szCurDir );

	ReadRegBinary( NULL, g_apszDriveRegID[ nDriveNum - 1 ], szNewDir, MAX_PATH, FALSE );
	if( strcmp( szNewDir, "Empty" ) && strcmp( szNewDir, "None" ) && strcmp( szNewDir, "Off" ) )
	{
		for( int i = strlen( szNewDir ); i > 0 && szNewDir[ i ] != '\\'; i-- );
		if( i > 0 )
			szNewDir[ i ] = '\0';
		SetCurrentDirectory( szNewDir );
	}

	if( g_ulScreenMode & SM_ATTR_SMALL_DLG )
	{
		CFileSmallDlg dlgFileSmall;
		if( dlgFileSmall.DoModal() == IDOK )
		{
			strName = dlgFileSmall.GetPathName();	
			SIO_Dismount( nDriveNum );
			SIO_Mount( nDriveNum, strName.GetBuffer( 0 ) );
			strName.ReleaseBuffer();
		}
	}
	else
	{
		char BASED_CODE szFilter[] = "Atari disk images (*.atr,*.xfd, etc)|*.atr;*.xfd;*.atr.gz;*.atz;*.xfd.gz;*.xfz;*.dcm|ATR Images only (*.atr,*.atr.gz,*.atz)|*.atr;*.atr.gz;*.atz|XFD Images only (*.xfd,*.xfd.gz,*.xfz)|*.xfd;*.xfd.gz;*.xfz|DCM Images only (*.dcm)|*.dcm|All Files (*.*)|*.*||";
		CFileDialog	fdlgDiskImage( TRUE, NULL, NULL, OFN_EXPLORER | OFN_FILEMUSTEXIST, szFilter, this );

		fdlgDiskImage.m_ofn.lpstrTitle = szPrompt;
		if( fdlgDiskImage.DoModal() == IDOK )
		{
			strName = fdlgDiskImage.GetPathName();
			SIO_Dismount( nDriveNum );
			SIO_Mount( nDriveNum, strName.GetBuffer( 0 ) );
			strName.ReleaseBuffer();
			WriteAtari800Drives( NULL );
		}
	}
	SetCurrentDirectory( szCurDir );

	ClearScreen( FALSE );
} /* #OF# CMainFrame::DriveInsert */

/*========================================================
Method   : CMainFrame::SaveState
=========================================================*/
/* #FN#
   Saves state of the Atari to a file */
void
/* #AS#
   Nothing */
CMainFrame::
SaveState( UINT unSaveRom )
{
	CString	strName = "";

	if( zlib_capable() == -1 )
	{
		MessageBox( "Cannot save/restore state files without a working ZLIB.DLL", "Atari800Win", MB_ICONEXCLAMATION | MB_OK );
		return;
	}

	if( unSaveRom )
	{
		int nResult = MessageBox( "It is not normally necessary to do this. It will save the Atari ROMS and BASIC in your state file, making it larger. The only reason this would be necessary is if you are running with patched ROMS/BASIC. Do you want to continue anyway?", "Atari800Win", MB_ICONQUESTION | MB_YESNO );
		if( nResult == IDNO )
			return;
	}
	if( g_ulScreenMode & SM_ATTR_SMALL_DLG )
	{
		CFileSmallDlg dlgFileSmall;
		if( dlgFileSmall.DoModal() == IDOK )
			strName = dlgFileSmall.GetPathName();	
	}
	else
	{
		char BASED_CODE szFilter[] = "Atari state images (*.a8s)|*.a8s;|All Files (*.*)|*.*||";
		CFileDialog	fdlgStateFile( FALSE, "a8s", "atarisav.a8s", OFN_EXPLORER | OFN_OVERWRITEPROMPT, szFilter, this );

		fdlgStateFile.m_ofn.lpstrTitle = "Select Atari saved-state file";
		if( fdlgStateFile.DoModal() == IDOK )
			strName = fdlgStateFile.GetPathName();
	}
	if( strName != "" )
	{
		int nResult = SaveAtariState( strName.GetBuffer( 0 ), "wb", unSaveRom );
		strName.ReleaseBuffer();
		if( nResult == FALSE )
		{
			MessageBox( "The Atari save state operation failed. You can check the 'File/View Log File' option to see a description of the problem.", "Atari800Win", MB_ICONEXCLAMATION | MB_OK );
		}
	}
	DescribeAtariSystem();
} /* #OF# CMainFrame::SaveState */

/*========================================================
Method   : CMainFrame::PreTranslateMessage
=========================================================*/
/* #FN#
   Translates window messages before they are dispatched to the
   TranslateMessage and DispatchMessage Windows functions */
BOOL
/* #AS#
   Nonzero if the message was translated and should not be dispatched;
   0 if the message was not translated and should be dispatched */
CMainFrame::
PreTranslateMessage( MSG *pMsg )
{
	WPARAM wp = pMsg->wParam;

	switch( pMsg->message )
	{
		/* This needs to be handled here instead of a OnXXX message because
		   we want to intercept control keys before they are parsed as
		   accelerators */
		case WM_KEYDOWN:
		{
			if( wp == (WPARAM)g_nCurrentVK )
				return TRUE;

			if( m_iToggleKeys[ wp ] )
			{
				if( ToggleKeyDown( wp ) )
					return TRUE;
			}

			if( (g_ulMiscStates & ATARI_USE_KEYTEMPLATE) && g_aucKeysCovered[ wp ] )
			{
				g_nNewKey = g_nKBTable[ wp ];
				
				if( g_nNewKey != AKEY_NONE )
				{
					if( SHIFT_KEY )
						g_nNewKey |= AKEY_SHFT;
					if( CTRL_KEY )
						g_nNewKey |= AKEY_CTRL;

					g_nNewVK = wp;
					return TRUE;
				}
				g_nNewVK = 0;
				return TRUE;
			}

			if( SHIFT_KEY && CTRL_KEY )
			{
				g_nNewKey = m_iShftCtrlKeys[ wp ];
				if( g_nNewKey != AKEY_NONE )
				{
					g_nNewVK = wp;
					return TRUE;
				}
				g_nNewVK = 0;
				return TRUE;
			}

			if( CTRL_KEY )
			{
				if( wp >= VK_F1 && wp <= VK_F8 )
				{
					DriveInsert( wp - VK_F1 + 1);
					return TRUE;
				}
				else
				{
					g_nNewKey = m_iCtrlKeys[ wp ];
					if( g_nNewKey != AKEY_NONE )
					{
						g_nNewVK = wp;
						return TRUE;
					}
				}
				g_nNewVK = 0;
				return TRUE;
			}

			if( SHIFT_KEY )
			{
				g_nNewKey = m_iShftKeys[ wp ];
				if( g_nNewKey != AKEY_NONE )
				{
					g_nNewVK = wp;
					return TRUE;
				}
				g_nNewVK = 0;
				return TRUE;
			}
			
			if( m_iNormKeys[ wp ] != AKEY_NONE )
			{
				g_nNewKey = m_iNormKeys[ wp ];
				if( g_nNewKey != AKEY_NONE )
				{
					g_nNewVK = wp;
					return TRUE;
				}
				g_nNewVK = 0;
				return TRUE;
			}
//			return CFrameWnd::PreTranslateMessage( pMsg );
			break;
		}

		case WM_SYSKEYDOWN:
		{
			if( wp == '\r' )
			{
				if( g_ulScreenMode & SM_MODE_FULL )
				{
					g_ulScreenMode &= ~SM_MODE_FULL;
					g_ulScreenMode |= SM_MODE_WIND;
				}
				else
				{
					_ASSERT(g_ulScreenMode & SM_MODE_WIND);

					g_ulScreenMode &= ~SM_MODE_WIND;
					g_ulScreenMode |= SM_MODE_FULL;
				}
				WriteRegDWORD( NULL, REG_DDRAW_MODE, g_ulScreenMode );
				InitialiseScreen();
				return TRUE;
			}
			break;
		}

		case WM_KEYUP:
		{
			if( m_iToggleKeys[ wp ] )
			{
				if( ToggleKeyUp( wp ) )
					return TRUE;
			}

			if( wp == (WPARAM)g_nCurrentVK )
			{
				g_nCurrentKey = AKEY_NONE;
				g_nCurrentVK  = 0;
				KEYPRESSED    = FALSE;
//				if( g_nNewKey != AKEY_NONE )
//					g_nNewVK |= AKEY_PENDING;
			}

			if( wp == (WPARAM)g_nNewVK )
			{
				if( !(g_nNewKey & SPECIAL_HANDLER_MASK) )
				{
					g_nNewKey = AKEY_NONE;
					g_nNewVK  = 0;
				}
			}
		}

		default:
			break;
	}
	return CFrameWnd::PreTranslateMessage( pMsg );
}


/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

/*========================================================
Method   : CMainFrame::OnCreate
=========================================================*/
/* #FN#
   Called by the framework when an application requests that the Windows
   window be created */
int
/* #AS#
   0 to continue the creation, 1 to destroy the Windows window */
CMainFrame::
OnCreate( LPCREATESTRUCT lpCreateStruct /* #IN# Pointer to a CREATESTRUCT
										        structure that contains
												information about the CWnd
												object being created */ )
{
#ifdef WIN_USE_PERF_COUNTER
	LARGE_INTEGER lnTimerRes;

	QueryPerformanceFrequency( &lnTimerRes ); 
	ASSERT(lnTimerRes.HighPart == 0);
	if( lnTimerRes.LowPart == 0 )
	{
		LoadString( NULL, IDS_NO_HIRES_TIMER, g_tszErrorString, LOADSTRING_STRING_SIZE );
		MessageBox( g_tszErrorString, NULL, MB_ICONSTOP | MB_OK );
		return FALSE;
	}
#endif /*WIN_USE_PERF_COUNTER*/

	if( CFrameWnd::OnCreate( lpCreateStruct ) == -1 )
		return -1;
	
	if( !m_wndStatusBar.Create( this ) ||
		!m_wndStatusBar.SetIndicators( indicators,
		  sizeof(indicators)/sizeof(UINT) ) )
	{
		TRACE0("Failed to create status bar\n");
		return -1; // fail to create
	}
	pStatusBar = (CWnd*)&m_wndStatusBar;
	
	return 0;
} /* #OF# CMainFrame::OnCreate */

/*========================================================
Method   : CMainFrame::OnFileReadState
=========================================================*/
/* #FN#
   Restores saved state file */
void
/* #AS#
   Nothing */
CMainFrame::
OnFileReadState()
{
	CString	strName = "";

	if( zlib_capable() == -1 )
	{
		MessageBox( "Cannot save/restore state files without a working ZLIB.DLL", "Atari800Win", MB_ICONEXCLAMATION | MB_OK );
		return;
	}

	if( g_ulScreenMode & SM_ATTR_SMALL_DLG )
	{
		CFileSmallDlg dlgFileSmall;
		if( dlgFileSmall.DoModal() == IDOK )
			strName = dlgFileSmall.GetPathName();	
	}
	else
	{
		char BASED_CODE szFilter[] = "Atari state images (*.a8s)|*.a8s;|All Files (*.*)|*.*||";
		CFileDialog	fdlgStateFile( TRUE, "a8s", "atarisav.a8s", OFN_EXPLORER | OFN_FILEMUSTEXIST, szFilter, this );

		fdlgStateFile.m_ofn.lpstrTitle = "Select Atari saved-state file";
		if( fdlgStateFile.DoModal() == IDOK )
			strName = fdlgStateFile.GetPathName();
	}
	if( strName != "" )
	{
		int nResult = ReadAtariState( strName.GetBuffer( 0 ), "rb" );
		strName.ReleaseBuffer();
		if( nResult == FALSE )
		{
			MessageBox( "The Atari read state operation failed. You can check the 'File/View Log File' option to see a description of the problem.", "Atari800Win", MB_ICONEXCLAMATION | MB_OK );
		}

		if( !(g_ulAtariState & ATARI_PAUSED) )
		{
			ClearScreen( FALSE );
			Toggle_Pause();
		}
		DescribeAtariSystem();
		return;
	}
	g_ulAtariState &= ~ATARI_PAUSED;
	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnFileReadState */

/*========================================================
Method   : CMainFrame::OnFileSaveState
=========================================================*/
/* #FN#
   Dumps the exact state of the Atari to a file */
void
/* #AS#
   Nothing */
CMainFrame::
OnFileSaveState() 
{
	SaveState( 0 );
} /* #OF# CMainFrame::OnFileSaveState */

/*========================================================
Method   : CMainFrame::OnFileSaveStateRoms
=========================================================*/
/* #FN#
   Saves a memory image with the ROMs */
void
/* #AS#
   Nothing */
CMainFrame::
OnFileSaveStateRoms() 
{
	SaveState( 1 );
} /* #OF# CMainFrame::OnFileSaveStateRoms */

/*========================================================
Method   : CMainFrame::OnFileDcmToAtr
=========================================================*/
/* #FN#
   Converts .DCM files to .ATR files */
void
/* #AS#
   Nothing */
CMainFrame::
OnFileDcmToAtr()
{
	char BASED_CODE szFilter[] = "DCM disk images (*.dcm)|*.dcm|All Files (*.*)|*.*||";
	char szLocalFileBuffer[ 8192 ];
	CFileDialog	dlgDiskImage( TRUE, NULL, NULL, OFN_PATHMUSTEXIST | OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_ALLOWMULTISELECT, szFilter, this );

	*szLocalFileBuffer = '\0';
	dlgDiskImage.m_ofn.lpstrFile  = szLocalFileBuffer;
	dlgDiskImage.m_ofn.nMaxFile   = 8192;
	dlgDiskImage.m_ofn.lpstrTitle = "Select DCM file(s) to convert";
	if( dlgDiskImage.DoModal() == IDOK  )
	{
		char     szDestDir[ MAX_PATH ];
		CString  strInput;
		FILE    *fpInput, *fpOutput;
		int      i;

		strcpy( szDestDir, dlgDiskImage.GetPathName() );
		for( i = strlen( szDestDir ); i > 0 && szDestDir[ i ] != '\\' && szDestDir[i] != '.'; i-- );
		if( i == 0 || szDestDir[ i ] != '.' )
			i = strlen( szDestDir );
		strcpy( &szDestDir[ i ], ".atr" );

		CFileDialog	dlgDestDir( FALSE, NULL, szDestDir, OFN_PATHMUSTEXIST | OFN_EXPLORER, NULL, this );
		dlgDestDir.m_ofn.lpstrTitle = "Select destination directory to write file(s)";
		if( dlgDestDir.DoModal() != IDOK )
			return;
		strcpy( szDestDir, dlgDestDir.GetPathName() );

		for( i = strlen( szDestDir ); i > 0 && szDestDir[ i ] != '\\'; i-- );
		szDestDir[ i ] = '\0';

		if( *szDestDir != '\0' )
		{
			POSITION pos = dlgDiskImage.GetStartPosition();

			ASSERT(pos);
			if( !pos )
				return;

			strInput = dlgDiskImage.GetNextPathName( pos );
			while( strInput != "" )
			{
				char szFullDestName[ MAX_PATH ];
				char szThisFileName[ MAX_PATH ];

				strcpy( szThisFileName, strInput.GetBuffer( 0 ) );
				strInput.ReleaseBuffer();
				fpInput = fopen( szThisFileName, "rb" );
				if( !fpInput )
				{
					char szError[ MAX_PATH + 32 ];
					sprintf( szError, "Can't open input file %s", szThisFileName );
					MessageBox( szError, "Atari800Win", MB_ICONEXCLAMATION | MB_OK );
					return;
				}
				for( i = strlen( szThisFileName ); i > 0 && szThisFileName[ i ]!= '\\'; i-- );
				if( szThisFileName[i] == '\\' )
					i++;

				strcpy( szFullDestName, szDestDir );
				strcat( szFullDestName, "\\" );
				strcat( szFullDestName, &szThisFileName[ i ] );

				for( i = strlen( szFullDestName ); i > 0 && szFullDestName[ i ] != '\\' && szFullDestName[ i ] != '.'; i-- );
				if( i == 0 || szFullDestName[ i ] != '.' )
					i = strlen( szFullDestName );
				strcpy( &szFullDestName[ i ], ".atr" );
				
				fpOutput = fopen( szFullDestName, "rb" );
				if( fpOutput )
				{
					char szWarning[ 256 + MAX_PATH ];
					int  nResult;

					sprintf( szWarning, "The file %s already exists, are you sure you want to overwrite it?", szFullDestName );
					nResult = MessageBox( szWarning, "Atari800Win", MB_ICONQUESTION | MB_YESNO );
					if( nResult == IDNO )
					{
						fclose( fpInput );
						return;
					}
					fclose( fpOutput );
				}
				fpOutput = fopen( szFullDestName, "wb" );
				if( !fpOutput )
				{
					char szWarning[ 256 + MAX_PATH ];
					sprintf( szWarning, "The file %s could not be opened for output, stopping operation", szFullDestName );
					MessageBox( szWarning, "Atari800Win", MB_ICONEXCLAMATION | MB_OK );
					return;
				}
				if( !dcmtoatr( fpInput, fpOutput, szThisFileName, szFullDestName ) )
				{
					int nResult;
					nResult = MessageBox( "There was an error during the operation and it was aborted. Would you like to check the error log for details?.", "Atari800Win", MB_ICONQUESTION | MB_YESNO );
					fclose( fpInput );
					fclose( fpOutput );
					if( nResult == IDYES )
					{
						CErrorLogDlg dlgErrorLog;
						dlgErrorLog.DoModal();
					}
					return;
				}
				fclose( fpOutput );
				fclose( fpInput );
				if( pos )
					strInput = dlgDiskImage.GetNextPathName( pos );
				else
					strInput = "";
			}
		}
	}
	MessageBox( "Conversion successfull", "Atari800Win", MB_ICONINFORMATION | MB_OK );

	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnFileDcmToAtr */

/*========================================================
Method   : CMainFrame::OnFileViewLog
=========================================================*/
/* #FN#
   Displays Error Log dialog box */
void
/* #AS#
   Nothing */
CMainFrame::
OnFileViewLog() 
{
	CErrorLogDlg dlgErrorLog;

	dlgErrorLog.DoModal();
	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnFileViewLog */

/*========================================================
Method   : CMainFrame::OnOptionsGraphics
=========================================================*/
/* #FN#
   Displays Graphics Options dialog box */
void
/* #AS#
   Nothing */
CMainFrame::
OnOptionsGraphics() 
{
	int	nResult;

	if( g_ulScreenMode & SM_ATTR_SMALL_DLG )
	{
		CGraphicsSmallDlg dlgGraphicsSmall( this );
		nResult = dlgGraphicsSmall.DoModal();
	}
	else
	{
		CGraphicsDlg dlgGraphics( this );
		nResult = dlgGraphics.DoModal();
	}
	/* Show the window because if we changed DDraw modes the frame will
	   be missing until forced to redraw here */
	if( nResult == IDOK )
		ShowWindow( SW_SHOWNORMAL );

	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnOptionsGraphics */

/*========================================================
Method   : CMainFrame::OnOptionsSound
=========================================================*/
/* #FN#
   Displays Sound Options dialog box */
void
/* #AS#
   Nothing */
CMainFrame::
OnOptionsSound() 
{
	CSoundDlg dlgSound;

	dlgSound.DoModal();
	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnOptionsSound */

/*========================================================
Method   : CMainFrame::OnOptionsJoystick
=========================================================*/
/* #FN#
   Displays Joystick Options dialog box */
void
/* #AS#
   Nothing */
CMainFrame::
OnOptionsJoystick() 
{
	CJoystickDlg dlgJoystick;

	dlgJoystick.DoModal();
	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnOptionsJoystick */

/*========================================================
Method   : CMainFrame::OnOptionsKeyboard
=========================================================*/
/* #FN#
   Displays Keyboard Templates dialog box */
void
/* #AS#
   Nothing */
CMainFrame::
OnOptionsKeyboard()
{
	CKeyboardDlg dlgKeyboard;

	dlgKeyboard.DoModal();
	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnOptionsKeyboard */

/*========================================================
Method   : CMainFrame::OnOptionsSoundFile
=========================================================*/
/* #FN#
   Saves the output of the Pokey processor to a file */
void
/* #AS#
   Nothing */
CMainFrame::
OnOptionsSoundFile()
{
	int nResult = IDOK;

	if( g_pfSndOutput )
		/* Close Sound Output file */
		CloseSndOutput();

	if( g_ulSoundState & SOUND_NOSOUND )
	{
		LoadString( NULL, IDS_SOUND_RECORD, g_tszErrorString, LOADSTRING_STRING_SIZE );
		MessageBox( g_tszErrorString, "Atari800Win", MB_ICONINFORMATION | MB_OK );
	}

	if( !(g_ulDontShowFlags & DONT_SHOW_SOUNDFILE_WARN) )
	{
		CWarningDlg	dlgWarning;
		char szMessage[ LOADSTRING_STRING_SIZE ];

		LoadString( NULL, IDS_WARNING_SOUNDFILE, szMessage, LOADSTRING_STRING_SIZE );
		dlgWarning.csWarnText = szMessage;
		dlgWarning.iWarnBit   = DONT_SHOW_SOUNDFILE_WARN;
		nResult = dlgWarning.DoModal();
	}

	if( nResult == IDOK )
	{
		char BASED_CODE szFilter[] = "Waveform audio data files (*.wav)|*.wav|All Files (*.*)|*.*||";
		CString strOutFileName;
 
		strOutFileName.GetBuffer( MAX_PATH );
		strOutFileName.Format( "%s", "atarisnd" );

		CFileDialog	fdlgTransScript( FALSE, "wav", strOutFileName, OFN_NOCHANGEDIR | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST, szFilter, this );
		fdlgTransScript.m_ofn.lpstrTitle = "Name of WAV file to save";

		if( fdlgTransScript.DoModal() == IDOK )
			/* Open Sound Output file */
			OpenSndOutput( (LPTSTR)(LPCTSTR)fdlgTransScript.GetPathName() );
	}
	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnOptionsSoundFile */

/*========================================================
Method   : CMainFrame::OnOptionsArtif
=========================================================*/
/* #FN#
   Displays Error Log dialog box */
void
/* #AS#
   Nothing */
CMainFrame::
OnOptionsArtif()
{
	global_artif_mode++;
	if( global_artif_mode > 4 )
		global_artif_mode = 0;
	art_main_init( 0, global_artif_mode );

	WriteRegDWORD( NULL, REG_ARTIF_MODE, global_artif_mode );
} /* #OF# CMainFrame::OnOptionsArtif */

/*========================================================
Method   : CMainFrame::OnAtariCartridge
=========================================================*/
/* #FN#
   Displays Cartridge Management dialog box */
void
/* #AS#
   Nothing */
CMainFrame::
OnAtariCartridge() 
{
	CCartridgeDlg dlgCartridge( this );

	dlgCartridge.DoModal();
	Invalidate();
	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnAtariCartridge */

/*========================================================
Method   : CMainFrame::OnAtariDisk
=========================================================*/
/* #FN#
   Displays Drive Selections dialog box */
void
/* #AS#
   Nothing */
CMainFrame::
OnAtariDisk()
{
	if( g_ulScreenMode & SM_ATTR_SMALL_DLG )
	{
		CDriveSmallDlg dlgDriveSmall( this );
		dlgDriveSmall.DoModal();
	}
	else
	{
		CDriveDlg dlgDrive( this );
		dlgDrive.DoModal();
	}
	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnAtariDisk */

/*========================================================
Method   : CMainFrame::OnAtariHarddisk
=========================================================*/
/* #FN#
   Displays Pick Virtual Harddisk Directories dialog box */
void
/* #AS#
   Nothing */
CMainFrame::
OnAtariHarddisk() 
{
	CHarddiskDlg dlgHardDisk;

	dlgHardDisk.DoModal();
	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnAtariHarddisk */

/*========================================================
Method   : CMainFrame::OnAtariHardware
=========================================================*/
/* #FN#
   Displays Hardware Settings dialog box */
void
/* #AS#
   Nothing */
CMainFrame::
OnAtariHardware()
{
	CHardwareDlg dlgHardware( this );

	dlgHardware.DoModal();
	Invalidate();
	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnAtariHardware */

/*========================================================
Method   : CMainFrame::OnAtarLoadExe
=========================================================*/
/* #FN#
   Load an Atari executable file directly */
void
/* #AS#
   Nothing */
CMainFrame::
OnAtarLoadExe()
{
	CString	strName = "";
	char	szCurDir[ MAX_PATH ];

	GetCurrentDirectory( MAX_PATH, szCurDir );
	SetCurrentDirectory( atari_exe_dir );

	if( g_ulScreenMode & SM_ATTR_SMALL_DLG )
	{
		CFileSmallDlg dlgFileSmall;
		if( dlgFileSmall.DoModal() == IDOK )
			strName = dlgFileSmall.GetPathName();	
	}
	else
	{
		char BASED_CODE szFilter[] = "All Files (*.*)|*.*|Atari executable files (*.com,*.exe,*.xex)|*.com;*.exe;*.xex||";
		CFileDialog	fdlgStateFile( TRUE, "xex", NULL, OFN_EXPLORER | OFN_FILEMUSTEXIST, szFilter, this );

		fdlgStateFile.m_ofn.lpstrTitle = "Select Atari executable file";
		if( fdlgStateFile.DoModal() == IDOK )
		{
			CString strNewPath;
			
			strName = fdlgStateFile.GetPathName();
			strNewPath = strName.Left( strName.ReverseFind( '\\' ) );
			/* Add ending backslash to drive name */
			if( strNewPath.GetLength() == 2 && strNewPath.GetAt( 1 ) == ':' )
				strNewPath += "\\";
			if( strNewPath.CompareNoCase( atari_exe_dir ) )
			{
				strcpy( atari_exe_dir, strNewPath );
				WriteRegString( NULL, REG_EXE_PATH, atari_exe_dir );
			}
		}
	}
	if( strName != "" )
	{
		int nResult;
		if( use_new_binload )
			nResult = BIN_loader( strName.GetBuffer( 0 ) );
		else
			nResult = ReadAtariExe( strName.GetBuffer( 0 ) );

		strName.ReleaseBuffer();
		if( nResult == FALSE )
			MessageBox( "Could not read Atari executable. You can check the 'File/View Log File' option to see a description of the problem.", "Atari800Win", MB_ICONEXCLAMATION | MB_OK );
	}
	SetCurrentDirectory( szCurDir );
	DescribeAtariSystem();
	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnAtarLoadExe */

/*========================================================
Method   : CMainFrame::OnAtariReboot
=========================================================*/
/* #FN#
   Restart the emulated Atari */
void
/* #AS#
   Nothing */
CMainFrame::
OnAtariReboot()
{
	Atari_ReInit();
	Invalidate();
} /* #OF# CMainFrame::OnAtariReboot */

/*========================================================
Method   : CMainFrame::OnMiscBenchmark
=========================================================*/
/* #FN#
   Renders speed of the current graphics mode */
void
/* #AS#
   Nothing */
CMainFrame::
OnMiscBenchmark()
{
	ScreenBenchmark();
	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnMiscBenchmark */

/*========================================================
Method   : CMainFrame::OnMiscSoundPokeySpeed
=========================================================*/
/* #FN#
   Tests the Pokey processor overhead with current settings */
void
/* #AS#
   Nothing */
CMainFrame::
OnMiscSoundPokeySpeed()
{
	ULONG  ulStartTime, ulTotalTime;
	char  *pszBuffer = NULL;
	int    nSampleSize;

	if( default_tv_mode == 1 )
		nSampleSize = g_nSoundRate / 50;
	else
		nSampleSize = g_nSoundRate / 60;
	pszBuffer = (char*)calloc( 1, nSampleSize + 1 );
	if( !pszBuffer )
		return;
	if( !(g_ulAtariState & ATARI_RUNNING) || (g_ulSoundState & SOUND_NOSOUND) )
	{
		LoadString( NULL, IDS_SOUND_TEST_WARNING, g_tszErrorString, LOADSTRING_STRING_SIZE );
		MessageBox( g_tszErrorString, "Atari800Win", MB_ICONINFORMATION | MB_OK );
		free( pszBuffer );
		return;
	}
	LoadString( NULL, IDS_SOUND_TEST_BEGIN, g_tszErrorString, LOADSTRING_STRING_SIZE );
	MessageBox( g_tszErrorString, "Atari800Win", MB_ICONINFORMATION | MB_OK );
	ulStartTime = timeGetTime();

	for( int i = 0; i < 60; i++ )
		Pokey_process( (unsigned char*)pszBuffer, nSampleSize );

	ulTotalTime = timeGetTime() - ulStartTime;
	LoadString( NULL, IDS_SOUND_TEST_RESULT, g_tszErrorString, LOADSTRING_STRING_SIZE );
	sprintf( pszBuffer, g_tszErrorString, nSampleSize, ulTotalTime, (double)ulTotalTime / 60.0 );
	MessageBox( pszBuffer, "Atari800Win", MB_ICONINFORMATION | MB_OK );
	free( pszBuffer );

	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnMiscSoundPokeySpeed */

/*========================================================
Method   : CMainFrame::OnMiscFileAssociations
=========================================================*/
/* #FN#
   Displays File Associations dialog box */
void
/* #AS#
   Nothing */
CMainFrame::
OnMiscFileAssociations()
{
	CFileAssociationsDlg dlgFileAssociations;

	dlgFileAssociations.DoModal();
	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnMiscFileAssociations */

/*========================================================
Method   : CMainFrame::OnMiscClearAll
=========================================================*/
/* #FN#
   Clears all saved settings/drives/ROMs */
void
/* #AS#
   Nothing */
CMainFrame::
OnMiscClearAll()
{
	int nResult;

	LoadString( NULL, IDS_CLEAR_AFFIRM, g_tszErrorString, LOADSTRING_STRING_SIZE );
	nResult = MessageBox( g_tszErrorString, "Atari800Win", MB_ICONQUESTION | MB_YESNO );

	if( nResult == IDYES )
	{
		DeleteAllRegKeys( NULL, REGNAME );
		HandleRegistry();
	}
} /* #OF# CMainFrame::OnMiscClearAll */

/*========================================================
Method   : CMainFrame::OnMiscMonitor
=========================================================*/
/* #FN#
   Launches monitor console */
void
/* #AS#
   Nothing */
CMainFrame::
OnMiscMonitor() 
{
	LaunchMonitor();
	ClearScreen( FALSE );
} /* #OF# CMainFrame::OnMiscMonitor */

/*========================================================
Method   : CMainFrame::OnTimer
=========================================================*/
/* #FN#
   The framework calls this member function after each interval specified
   in the SetTimer member function used to install a timer */
void
/* #AS#
   Nothing */
CMainFrame::
OnTimer( UINT nIDEvent /* #IN# Specifies the identifier of the timer */ )
{
	if( nIDEvent == TIMER_READ_JOYSTICK )
	{
		ReadJoystickInput( 0, 0 );
		ReadJoystickInput( 1, 0 );
		ReadJoystickInput( 2, 0 );
		ReadJoystickInput( 3, 0 );
		if( !(g_ulAtariState & ATARI_PAUSED) )
			::KillTimer( g_hMainWnd, TIMER_READ_JOYSTICK );
	}
	CFrameWnd::OnTimer( nIDEvent );
} /* #OF# CMainFrame::OnTimer */

/*========================================================
Method   : CMainFrame::OnMove
=========================================================*/
/* #FN#
   The framework calls this function after the CWnd object has been moved */
void
/* #AS#
   Nothing */
CMainFrame::
OnMove( int x, /* #IN# New x-coordinate location of the upper-left corner */
	    int y  /* #IN# New y-coordinate location of the upper-left corner */ )
{
	CFrameWnd::OnMove( x, y );

	if( g_hWnd )
		ComputeClipArea();
} /* #OF# CMainFrame::OnMove */

/*========================================================
Method   : CMainFrame::OnClose
=========================================================*/
/* #FN#
   The framework calls this function as a signal that the CWnd or an
   application is to terminate */
void
/* #AS#
   Nothing */
CMainFrame::
OnClose() 
{
	RECT rc;

	if( g_pfSndOutput )
		/* Close Sound Output file */
		CloseSndOutput();

	GetWindowRect( &rc );

	if( (rc.left != 0) && (g_nStartX != rc.left) )
		WriteRegDWORD( NULL, REG_START_XPOS, (DWORD)rc.left );

	if( (rc.top != 0) && (g_nStartY != rc.top) )
		WriteRegDWORD( NULL, REG_START_YPOS, (DWORD)rc.top );
	
	CFrameWnd::OnClose();
} /* #OF# CMainFrame::OnClose */
